Method: Webgen::Node#method_missing
- Defined in:
- lib/webgen/node.rb
#method_missing(name, *args, &block) ⇒ Object (private)
Delegate missing methods to a processor. The current node is placed into the argument array as the first argument before the method name is invoked on the processor.
376 377 378 379 380 381 382 |
# File 'lib/webgen/node.rb', line 376 def method_missing(name, *args, &block) if node_info[:processor] website.cache.instance(node_info[:processor]).send(name, *([self] + args), &block) else super end end |